@@ -1260,15 +1260,19 @@ class Backtest:
         broker: _Broker = self._broker(data=data)
         strategy: Strategy = self._strategy(broker, data, kwargs)
 
-        strategy.init()
-        data._update()  # Strategy.init might have changed/added to data.df
+        def prepare_strategy():
+            strategy.init()
+            data._update()  # Strategy.init might have changed/added to data.df
+            # Indicators used in Strategy.next()
+            indicator_attrs = {
+                attr: indicator
+                for attr, indicator in strategy.__dict__.items()
+                if isinstance(indicator, _Indicator)
+            }.items()
 
-        # Indicators used in Strategy.next()
-        indicator_attrs = {
-            attr: indicator
-            for attr, indicator in strategy.__dict__.items()
-            if isinstance(indicator, _Indicator)
-        }.items()
+            return indicator_attrs
+
+        indicator_attrs = prepare_strategy()
 
         # Skip first few candles where indicators are still "warming up"
         # +1 to have at least two entries available
@@ -1364,7 +1368,6 @@ class Backtest:
         method: str = "grid",
         max_tries: Union[int, float] = None,
         constraint: Callable[[dict], bool] = None,
-        division_size: int = 5,
         **kwargs,
     ) -> Union[
         pd.Series, Tuple[pd.Series, pd.Series], Tuple[pd.Series, pd.Series, dict]
@@ -1373,15 +1376,19 @@ class Backtest:
         broker: _Broker = self._broker(data=data)
         strategy: Strategy = self._strategy(broker, data, dict([]))
 
-        strategy.init()
-        data._update()  # Strategy.init might have changed/added to data.df
+        def prepare_strategy():
+            strategy.init()
+            data._update()  # Strategy.init might have changed/added to data.df
+            # Indicators used in Strategy.next()
+            indicator_attrs = {
+                attr: indicator
+                for attr, indicator in strategy.__dict__.items()
+                if isinstance(indicator, _Indicator)
+            }.items()
 
-        # Indicators used in Strategy.next()
-        indicator_attrs = {
-            attr: indicator
-            for attr, indicator in strategy.__dict__.items()
-            if isinstance(indicator, _Indicator)
-        }.items()
+            return indicator_attrs
+
+        indicator_attrs = prepare_strategy()
 
         # Skip first few candles where indicators are still "warming up"
         # skip to initial out sample portion of data
@@ -1414,7 +1421,7 @@ class Backtest:
                             **kwargs,
                         )
                         strategy.update_params(Optimize[1])
-                        strategy.init()
+                        indicator_attrs = prepare_strategy()
 
                 # Prepare data and indicators for `next` call
                 data._set_length(i + 1)
@@ -1454,9 +1461,116 @@ class Backtest:
                 end_data_portion=1,
                 **kwargs,
             )
+            strategy.update_params(FinalOptimize[1])
 
             self._results = self._compute_stats(broker, strategy)
-        return (self._results, FinalOptimize[1])
+        return self._results
+
+    # Cross-Validation Optimization
+    def cross_validation_optimization(
+        self,
+        *,
+        maximize: Union[str, Callable[[pd.Series], float]] = "SQN",
+        method: str = "grid",
+        max_tries: Union[int, float] = None,
+        constraint: Callable[[dict], bool] = None,
+        **kwargs,
+    ) -> Union[
+        pd.Series, Tuple[pd.Series, pd.Series], Tuple[pd.Series, pd.Series, dict]
+    ]:
+        data = _Data(self._data.copy(deep=False))
+        broker: _Broker = self._broker(data=data)
+        strategy: Strategy = self._strategy(broker, data, dict([]))
+
+        def prepare_strategy():
+            strategy.init()
+            data._update()  # Strategy.init might have changed/added to data.df
+            # Indicators used in Strategy.next()
+            indicator_attrs = {
+                attr: indicator
+                for attr, indicator in strategy.__dict__.items()
+                if isinstance(indicator, _Indicator)
+            }.items()
+
+            return indicator_attrs
+
+        indicator_attrs = prepare_strategy()
+
+        # Skip first few candles where indicators are still "warming up"
+        # skip to initial out sample portion of data
+        # +1 to have at least two entries available
+        start = 1 + max(
+            max(
+                (
+                    np.isnan(indicator.astype(float)).argmin(axis=-1).max()
+                    for _, indicator in indicator_attrs
+                ),
+                default=0,
+            ),
+            int(len(self._data) * 0.1) - 1,
+        )
+
+        # Disable "invalid value encountered in ..." warnings. Comparison
+        # np.nan >= 3 is not invalid; it's False.
+        with np.errstate(invalid="ignore"):
+            splits = [0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9]
+            for i in range(start, len(self._data)):
+                for s in splits:
+                    if int(s * len(self._data)) == i:
+                        Optimize = self.optimize(
+                            maximize=maximize,
+                            method=method,
+                            max_tries=max_tries,
+                            constraint=constraint,
+                            start_data_portion=0,
+                            end_data_portion=s,
+                            **kwargs,
+                        )
+                        strategy.update_params(Optimize[1])
+                        indicator_attrs = prepare_strategy()
+
+                # Prepare data and indicators for `next` call
+                data._set_length(i + 1)
+                for attr, indicator in indicator_attrs:
+                    # Slice indicator on the last dimension (case of 2d indicator)
+                    setattr(strategy, attr, indicator[..., : i + 1])
+
+                # Handle orders processing and broker stuff
+                try:
+                    broker.next()
+                except _OutOfMoneyError:
+                    break
+
+                # Next tick, a moment before bar close
+                strategy.next()
+            else:
+                # Close any remaining open trades so they produce some stats
+                for trade in broker.trades:
+                    trade.close()
+
+                # Re-run broker one last time to handle orders placed in the last strategy
+                # iteration. Use the same OHLC values as in the last broker iteration.
+                if start < len(self._data):
+                    try_(broker.next, exception=_OutOfMoneyError)
+
+            # Set data back to full length
+            # for future `indicator._opts['data'].index` calls to work
+            data._set_length(len(self._data))
+
+            # Final Optimization
+            FinalOptimize = self.optimize(
+                maximize=maximize,
+                method=method,
+                max_tries=max_tries,
+                constraint=constraint,
+                start_data_portion=0,
+                end_data_portion=1,
+                **kwargs,
+            )
+            strategy.update_params(FinalOptimize[1])
+
+            self._results = self._compute_stats(broker, strategy)
+        return self._results
 
     def optimize(
         self,
@@ -1720,7 +1834,7 @@ class Backtest:
                 from skopt import forest_minimize
                 from skopt.space import Integer, Real, Categorical
                 from skopt.utils import use_named_args
-                from skopt.callbacks import DeltaXStopper
+                from skopt.callbacks import EarlyStopper
                 from skopt.learning import ExtraTreesRegressor
             except ImportError:
                 raise ImportError(
@@ -1782,6 +1896,31 @@ class Backtest:
                     return INVALID
                 return value
 
+            class XYStopper(EarlyStopper):
+                def __init__(self):
+                    super(EarlyStopper, self).__init__()
+                    self.n_best = 5
+
+                def _criterion(self, result):
+                    # Y Check
+                    if len(result.func_vals) >= self.n_best:
+                        func_vals = np.sort(result.func_vals)
+                        worst = func_vals[self.n_best - 1]
+                        best = func_vals[0]
+
+                        return best / worst > 0.5
+                    # X Check
+                    if len(result.x_iters) >= 2:
+                        return (
+                            result.space.distance(
+                                result.x_iters[-2], result.x_iters[-1]
+                            )
+                            < 5
+                        )
+
+                    else:
+                        return None
+
             with warnings.catch_warnings():
                 warnings.filterwarnings(
                     "ignore", "The objective has been evaluated at this point before."
@@ -1798,7 +1937,7 @@ class Backtest:
                     kappa=3,
                     n_initial_points=min(max_tries, 20 + 3 * len(kwargs)),
                     initial_point_generator="lhs",  # 'sobel' requires n_initial_points ~ 2**N
-                    callback=DeltaXStopper(9e-7),
+                    callback=XYStopper(),
                     random_state=random_state,
                 )
 
